//----------------------------------------------------------------------
//
//  FILE:   BenchmarkPlot.cpp
//  AUTHOR: Ian Ross
//  DATE:   28-MAR-2005
//
//----------------------------------------------------------------------
//
//  Base class for MOTIF benchmark plot generation classes.
//
//----------------------------------------------------------------------

//----------------------------------------------------------------------
//
//  HEADER FILES
//

// Standard headers.

#include <cstdio>
#include <sys/types.h>
#include <unistd.h>
#include <time.h>


// Local headers.

#include "BenchmarkPlot.hh"
#include "DBFile.hh"
#include "TempFile.hh"


//----------------------------------------------------------------------
//
//  MEMBER FUNCTION DEFINITIONS
//

// Only created via derived classes.

BenchmarkPlot::BenchmarkPlot(string model, Domain domain,
                             string var_name, string var_full_name,
                             OutputFreq freq, const PlotSetup &setup) :
  _model(model), _domain(domain), _var_name(var_name),
  _var_full_name(var_full_name),
  _freq(freq), _setup(setup), _valid(false), _eps_file(TempFile::get("eps"))
{
  // Set up data file - either just point straight to the required
  // input data file, or generate an anomaly file.  In both cases,
  // this is handled using the DBFile class, which finds appropriate
  // MOTIF database entries, and is also able to collapse multiple
  // timeslices into a single file, either by concatenation (for
  // daily, monthly or annual data) or by month-by-month averaging
  // (for seasonal data).

  _data_file = "";
  if (_setup.single()) {
    // Simple non-anomaly data.

    DBFile db_file(_model, _domain, _var_name, _setup.run(), _freq);
    if (db_file.valid()) {
      if (db_file.multiple()) db_file.collapse_multiple();
      _data_file = db_file.full_path();
    }
  } else {
    // Anomaly data.

    DBFile db_file1(_model, _domain, _var_name, _setup.experiment(), _freq);
    DBFile db_file2(_model, _domain, _var_name, _setup.control(), _freq);
    if (db_file1.valid() && db_file2.valid()) {
      if (db_file1.multiple()) db_file1.collapse_multiple();
      if (db_file2.multiple()) db_file2.collapse_multiple();
      _data_file = db_file1.generate_anomalies(db_file2);
    }
  }
}


string BenchmarkPlot::date_mark(void)
{
  time_t t = time(0);
  struct tm *tim = localtime(&t);
  char buff[32];
  strftime(buff, 32, "%e %b %Y" , tim);
  return buff;
}


string BenchmarkPlot::get_data_file(Domain domain, string var_name)
{
  string data_file = "";
  if (_setup.single()) {
    // Simple non-anomaly data.

    DBFile db_file(_model, domain, var_name, _setup.run(), _freq);
    if (db_file.valid()) {
      if (db_file.multiple()) db_file.collapse_multiple();
      data_file = db_file.full_path();
    }
  } else {
    // Anomaly data.

    DBFile db_file1(_model, domain, var_name, _setup.experiment(), _freq);
    DBFile db_file2(_model, domain, var_name, _setup.control(), _freq);
    if (db_file1.valid() && db_file2.valid()) {
      if (db_file1.multiple()) db_file1.collapse_multiple();
      if (db_file2.multiple()) db_file2.collapse_multiple();
      data_file = db_file1.generate_anomalies(db_file2);
    }
  }
  return data_file;
}


void BenchmarkPlot::map_plot(string real_data_file)
{
  FILE *fp = popen("ncl", "w");
//   FILE *fp = fopen(TempFile::get("ncl").c_str(), "w");
  string outfile = TempFile::get("ps");
  string outfile2 = outfile.substr(0, outfile.size() - 3);

  fprintf(fp, "load \"$NCARG_ROOT/lib/ncarg/nclex/gsun/gsn_code.ncl\"\n");
  fprintf(fp, "load \"$NCARG_ROOT/lib/ncarg/nclex/gsun/gsn_csm.ncl\"\n");
  fprintf(fp, "begin\n");
  fprintf(fp, "fp = addfile(\"%s\", \"r\")\n", real_data_file.c_str());
  fprintf(fp, "var = fp->%s\n", _var_name.c_str());
  fprintf(fp, "lat = fp->lat\n");
  fprintf(fp, "lon = fp->lon\n");
  fprintf(fp, "var@nlon = dimsizes(var&lon)\n");
  fprintf(fp, "var@nlat = dimsizes(var&lat)\n");
  fprintf(fp, "type = \"ps\"\n");
  fprintf(fp, "type@wkOrientation = \"landscape\"\n");
  fprintf(fp, "type@wkDeviceLowerY = 18 - 10\n");
  fprintf(fp, "type@wkDeviceUpperY = 792 - 18 - 10\n");
  fprintf(fp, "type@wkDeviceLowerX = 18 - 120\n");
  fprintf(fp, "type@wkDeviceUpperX = 792 - 18 - 120\n");
  fprintf(fp, "wks = gsn_open_wks(type, \"%s\")\n", outfile2.c_str());
  fprintf(fp, "res = True\n");
  int grid_colour;
  colours(fp, grid_colour);
  float scale_factor, offset;
  string units;
  scaling(scale_factor, offset, units);
  fprintf(fp, "res@sfXCStartV = lon(0)\n");
  fprintf(fp, "res@sfXCEndV   = lon(0) + 360.0\n");
  fprintf(fp, "res@sfYCStartV = lat(0)\n");
  fprintf(fp, "res@sfYCEndV   = lat(var@nlat - 1)\n");
  fprintf(fp, "res@cnFillOn = True\n");
  fprintf(fp, "res@cnFillColors = fill_colours\n");
  fprintf(fp, "res@cnLevelSelectionMode = \"ExplicitLevels\"\n");
  fprintf(fp, "res@lbLabelFont = \"Helvetica\"\n");
  fprintf(fp, "res@cnLevels = levels\n");
  fprintf(fp, "res@cnMonoFillPattern = True\n");
  fprintf(fp, "res@cnMonoFillColor = False\n");
  fprintf(fp, "res@cnLineLabelsOn = False\n");
  fprintf(fp, "res@cnInfoLabelOn = False\n");
  fprintf(fp, "res@cnLinesOn = False\n");
  fprintf(fp, "res@sfYArray = lat\n");
  fprintf(fp, "res@vpYF = 1.00\n");
  fprintf(fp, "res@vpXF = 0.05\n");
  fprintf(fp, "res@vpWidthF = 0.90\n");
  fprintf(fp, "res@vpHeightF = 0.90\n");
  fprintf(fp, "res@sfMissingValueV = 1.0E20\n");
  fprintf(fp, "res@mpGridAndLimbOn = True\n");
  fprintf(fp, "res@mpGridLineColor = %d\n", grid_colour);
  fprintf(fp, "res@mpLimitMode = \"LatLon\"\n");
  fprintf(fp, "res@mpMinLonF = -180.0\n");
  fprintf(fp, "res@mpMaxLonF = 180.0\n");
  fprintf(fp, "res@mpMinLatF = -90.0\n");
  fprintf(fp, "res@mpMaxLatF = 90.0\n");
  fprintf(fp, "res@mpPerimOn = True\n");
  fprintf(fp, "res@mpPerimDrawOrder = \"PostDraw\"\n");
  fprintf(fp, "res@pmTickMarkDisplayMode = \"Always\"\n");
  fprintf(fp, "res@tmXBLabelFont = \"Helvetica\"\n");
  fprintf(fp, "res@tmYLLabelFont = \"Helvetica\"\n");
  fprintf(fp, "res@pmLabelBarDisplayMode = \"Always\"\n");
  fprintf(fp, "res@lbLabelBarOn = True\n");
  fprintf(fp, "res@lbTitleString = \"%s\"\n", units.c_str());
  fprintf(fp, "res@lbTitleFont = \"Helvetica\"\n");
  fprintf(fp, "res@lbTitleFontHeightF = 0.015\n");
  fprintf(fp, "res@lbTitlePosition = \"Bottom\"\n");
  fprintf(fp, "res@lbTitleOn = True\n");
  fprintf(fp, "res@tmXBLabelFontHeightF = 0.01\n");
  fprintf(fp, "res@tmXBMajorLengthF = 0.005\n");
  fprintf(fp, "res@mpGridAndLimbOn = True\n");
  fprintf(fp, "res@mpGridSpacingF = 30\n");
  fprintf(fp, "res@gsnStringFont = \"Helvetica\"\n");
  fprintf(fp, "res@gsnLeftString = \"%s\"\n", title_string().c_str());
  fprintf(fp, "res@gsnRightString = \"%s\"\n", date_mark().c_str());
  fprintf(fp, "plot = gsn_csm_contour_map(wks, %f * var(0,:,:) + %f, res)\n",
          scale_factor, offset);
  fprintf(fp, "end\n");
  pclose(fp);

  string command = string("ps2eps -q ") + outfile;
  system(command.c_str());
  rename((outfile2 + ".eps").c_str(), eps_file().c_str());
}


//----------------------------------------------------------------------
//----------------------------------------------------------------------
//----------------------------------------------------------------------
