//----------------------------------------------------------------------
//
//  FILE:   DBFile.cpp
//  AUTHOR: Ian Ross
//  DATE:   28-MAR-2005
//
//----------------------------------------------------------------------
//
//  Class for handling MOTIF database files for benchmarking tools.
//
//----------------------------------------------------------------------

//----------------------------------------------------------------------
//
//  HEADER FILES
//

// Standard headers.

#include <cstdlib>
#include <unistd.h>
#include <glob.h>

using namespace std;


// Local headers.

#include "DBFile.hh"
#include "TempFile.hh"


//----------------------------------------------------------------------
//
//  MEMBER FUNCTION DEFINITIONS
//

DBFile::DBFile(string model, Domain domain, string var_name,
               ModelRun run, OutputFreq freq) :
  _model(model), _domain(domain), _var_name(var_name), _run(run), _freq(freq),
  _valid(false), _multiple(false), _collapsed(false), _constructed(false)
{
  build_path_template();
  string cons_test_path = _constructed_path_template + ".nc";
  if (!access(cons_test_path.c_str(), R_OK)) {
    _valid = true;
    _constructed = true;
  } else {
    string test_path = _path_template + ".nc";
    if (!access(test_path.c_str(), R_OK))
      _valid = true;
    else {
      string test_pattern =
        _path_template + "_[0-9][0-9][0-9][0-9]-[0-9][0-9][0-9][0-9].nc";
      glob_t globs;
      glob(test_pattern.c_str(), 0, 0, &globs);
      if (globs.gl_pathc > 0) {
        _valid = true;
        _multiple = true;
      }
      globfree(&globs);
    }
  }
//   cout << "DBFile: " << model << " " << var_name << endl;
//   cout << "  " << _path_template << endl;
//   cout << "  valid=" << _valid
//        << "  cons=" << _constructed
//        << "  mult=" << _multiple << endl;
}

string DBFile::full_path(void) const
{
  if (_constructed)
    return _constructed_path_template + ".nc";
  else {
    if (!_multiple) {
      if (_valid)
        return _path_template + ".nc";
      else
        throw "DBFile::full_path - No data";
    } else {
      if (_collapsed)
        return _collapsed_path;
      else
        throw "DBFile::full_path - Multiple files; not collapsed";
    }
  }
}


void DBFile::collapse_multiple(void)
{
  if (!_multiple) return;
  string months[12];
  for (int idx = 0; idx < 12; ++idx) {
    char buff[512];
    sprintf(buff, "ncra -d time,%d,,12 %s_"
            "[0-9][0-9][0-9][0-9]-[0-9][0-9][0-9][0-9].nc ###",
            idx, _path_template.c_str());
    months[idx] = TempFile::get("nc", buff);
  }
  string command = "ncrcat ";
  for (int idx = 0; idx < 12; ++idx)
    command += months[idx] + ' ';
  command += "###";
  _collapsed_path = TempFile::get("nc", command);
  _collapsed = true;
}


string DBFile::generate_anomalies(DBFile &other)
{
  if (_model != other._model || _domain != other._domain ||
      _var_name != other._var_name || _freq != other._freq)
    throw "DBFile::generate_anomalies - incompatible data";

  string command = "ncdiff " + full_path() + " " + other.full_path() + " ###";
  return TempFile::get("nc", command);
}

#if 0
void DBFile::build_path_template(void)
{
  _path_template = "/home/ggxir/foam-data/";
  switch (_run) {
  case OA_0K:    _path_template += "OA_MOD";   break;
  case OAV_0K:   _path_template += "OAV_MOD";  break;
  case OA_6K:    _path_template += "OA_6K";    break;
  case OAV_6K:   _path_template += "OAV_6K";   break;
  case OA_21K:   _path_template += "OA_21K";   break;
  case OAV_21K:  _path_template += "OAV_21K";  break;
  }
  _path_template += "/motif/";
  switch (_domain) {
  case ATMOS:    _path_template += "A_";  break;
  case OCEAN:    _path_template += "O_";  break;
  case SURFACE:  _path_template += "S_";  break;
  case ICE:      _path_template += "I_";  break;
  }
  switch (_freq) {
  case DAILY:     _path_template += "DA/";  break;
  case MONTHLY:   _path_template += "MO/";  break;
  case SEASONAL:  _path_template += "SE/";  break;
  case ANNUAL:    _path_template += "AN/";  break;
  case FIXED:     _path_template += "FX/";  break;
  }
  _path_template += _var_name;
  switch (_domain) {
  case ATMOS:    _path_template += "_A_";  break;
  case OCEAN:    _path_template += "_O_";  break;
  case SURFACE:  _path_template += "_S_";  break;
  case ICE:      _path_template += "_I_";  break;
  }
  switch (_freq) {
  case DAILY:     _path_template += "DA_";  break;
  case MONTHLY:   _path_template += "MO_";  break;
  case SEASONAL:  _path_template += "SE_";  break;
  case ANNUAL:    _path_template += "AN_";  break;
  case FIXED:     _path_template += "FX_";  break;
  }
  _path_template += "PMIP2_";
  switch (_run) {
  case OA_0K:    _path_template += "0K_OA_";    break;
  case OAV_0K:   _path_template += "0K_OAV_";   break;
  case OA_6K:    _path_template += "6K_OA_";    break;
  case OAV_6K:   _path_template += "6K_OAV_";   break;
  case OA_21K:   _path_template += "21K_OA_";   break;
  case OAV_21K:  _path_template += "21K_OAV_";  break;
  }
  _path_template += _model;
}
#endif


void DBFile::build_path_template(void)
{
  string tmp = "pmip2_";
  switch (_run) {
  case OA_0K:    tmp += "0k_oa/";    break;
  case OAV_0K:   tmp += "0k_oav/";   break;
  case OA_6K:    tmp += "6k_oa/";    break;
  case OAV_6K:   tmp += "6k_oav/";   break;
  case OA_21K:   tmp += "21k_oa/";   break;
  case OAV_21K:  tmp += "21k_oav/";  break;
  }
  switch (_domain) {
  case ATMOS:    tmp += "atm/";   break;
  case OCEAN:    tmp += "ocn/";   break;
  case SURFACE:  tmp += "land/";  break;
  case ICE:      tmp += "ice/";   break;
  }
  switch (_freq) {
  case DAILY:     tmp += "da/";     break;
  case MONTHLY:   tmp += "mo/";     break;
  case SEASONAL:  tmp += "se/";     break;
  case ANNUAL:    tmp += "an/";     break;
  case FIXED:     tmp += "fixed/";  break;
  }
  tmp += _var_name;
  tmp += "/";
  tmp += _var_name;
  switch (_domain) {
  case ATMOS:    tmp += "_A_";  break;
  case OCEAN:    tmp += "_O_";  break;
  case SURFACE:  tmp += "_S_";  break;
  case ICE:      tmp += "_I_";  break;
  }
  switch (_freq) {
  case DAILY:     tmp += "DA_";  break;
  case MONTHLY:   tmp += "MO_";  break;
  case SEASONAL:  tmp += "SE_";  break;
  case ANNUAL:    tmp += "AN_";  break;
  case FIXED:     tmp += "FX_";  break;
  }
  tmp += "pmip2_";
  switch (_run) {
  case OA_0K:    tmp += "0k_oa_";    break;
  case OAV_0K:   tmp += "0k_oav_";   break;
  case OA_6K:    tmp += "6k_oa_";    break;
  case OAV_6K:   tmp += "6k_oav_";   break;
  case OA_21K:   tmp += "21k_oa_";   break;
  case OAV_21K:  tmp += "21k_oav_";  break;
  }
  tmp += _model;

  _constructed_path_template = "/mnt/hgfs/pmip2/Share/ian_tmp/seasonal-data/";
  _constructed_path_template += tmp;
  _path_template = "/mnt/hgfs/pmip2/pmip2db/";
  _path_template += tmp;
}


//----------------------------------------------------------------------
//----------------------------------------------------------------------
//----------------------------------------------------------------------
