//----------------------------------------------------------------------
//
//  FILE:   PrecipBenchmarkPlot.cpp
//  AUTHOR: Ian Ross
//  DATE:   28-MAR-2005
//
//----------------------------------------------------------------------
//
//  Class for MOTIF benchmark seasonal plot generation.
//
//----------------------------------------------------------------------

//----------------------------------------------------------------------
//
//  HEADER FILES
//

// Standard headers.

#include <iostream>
#include <fstream>
#include <cstdio>
#include <netcdf.hh>

using namespace std;


// Local headers.

#include "PrecipBenchmarkPlot.hh"
#include "TempFile.hh"


//----------------------------------------------------------------------
//
//  MEMBER FUNCTION DEFINITIONS
//

PrecipBenchmarkPlot::PrecipBenchmarkPlot
(string model, const PlotSetup &setup, Component comp) :
  BenchmarkPlot(model, ATMOS, "pr", "Total Precipitation", SEASONAL, setup),
  _component(comp)
{
  if (data_file() != "") {
    if (_component == ANNUAL || _component == ZONAL_AFRICA) {
      char buff[256];
      sprintf(buff, "ncra -d time,0,,1 %s ###", data_file().c_str());
      _plot_data_file = TempFile::get("nc", buff);
      if (_component == ZONAL_AFRICA) {
        // Zonally average from 20W to 30E for each latitude.

        string tmp_file = _plot_data_file;
        sprintf(buff, "ncks -d lat,0.0,30.0 -d lon,-20.0,30.0 %s ###",
                tmp_file.c_str());
        tmp_file = TempFile::get("nc", buff);
        sprintf(buff, "ncwa -a lon %s ###", tmp_file.c_str());
        _plot_data_file = TempFile::get("nc", buff);
      }
    } else if (_component == DJF || _component == JJA) {
      char buff[256];
      int start, end;
      switch (_component) {
      case DJF: start = 11; end = 1; break;
      case JJA: start = 5;  end = 7; break;
      }
      sprintf(buff, "ncks -d time,%d,%d %s ###",
              start, end, data_file().c_str());
      string tmp_file = TempFile::get("nc", buff);
      sprintf(buff, "ncra -d time,0,2 %s ###", tmp_file.c_str());
      _plot_data_file = TempFile::get("nc", buff);
    } else {
    }
  } else
    _plot_data_file = "";
}


void PrecipBenchmarkPlot::plot(void)
{
  if (_plot_data_file == "") return;
  if (_component != ZONAL_AFRICA)
    map_plot(_plot_data_file);
  else
    zonally_averaged_plot(_plot_data_file);
  set_valid();
}


void PrecipBenchmarkPlot::zonally_averaged_plot(string data_file)
{
  NcFile prfp(data_file.c_str());
  NcDim *lat_dim = prfp.get_dim("lat");
  float *lat = new float[lat_dim->size()], *pr = new float[lat_dim->size()];
  NcVar *lat_var = prfp.get_var("lat");
  NcVar *pr_var = prfp.get_var("pr");
  for (int idx = 0; idx < lat_dim->size(); ++idx) {
    lat[idx] = lat_var->as_float(idx);
    pr[idx] = pr_var->as_float(idx);
  }
  string dat_file = TempFile::get("dat");
  ofstream outstr(dat_file.c_str());
  for (int idx = 0; idx < lat_dim->size(); ++idx)
    outstr << lat[idx] << ' ' << pr[idx] << endl;
  outstr.close();
  delete [] lat;
  delete [] pr;
  FILE *fp = popen("gnuplot", "w");
  fprintf(fp, "set term postscript eps enhanced monochrome "
          "solid \"Helvetica\" 18\n");
  fprintf(fp, "set output '%s'\n", eps_file().c_str());
  fprintf(fp, "set data style lines\n");
  fprintf(fp, "set nokey\n");
  fprintf(fp, "set title '%s Zonally Averaged Africa Precipitation (%s)'\n",
          _model.c_str(), string(_setup).c_str());
  fprintf(fp, "set ylabel 'Precipitation (mm/day)'\n");
  fprintf(fp, "set xlabel 'Latitude ({/Symbol \260}N)'\n");
  fprintf(fp, "plot '%s' using 1:($2 * 3600 * 24) with lines lw 4\n",
          dat_file.c_str());
  fprintf(fp, "quit\n");
  pclose(fp);
}


void PrecipBenchmarkPlot::colours(FILE *fp, int &grid_colour)
{
  grid_colour = 254;
  fprintf(fp, "gsn_define_colormap(wks, \"BlueRedGray\")\n");

  if (_setup.single()) {
    fprintf(fp, "levels = (/ 2, 4, 8, 12, 16, 30 /)\n");
    fprintf(fp, "fill_colours = (/ 0, 79, 64, 48, 33, 17, 2 /)\n");
  } else {
    fprintf(fp, "levels = "
            "(/ -20,-18,-16,-14,-12,-10,-8,-6,-4,-2,-1,-0.5,-0.25,-0.01,"
            "0.01,0.25,0.5,1,2,4,6,8,10,12,14,16,18,20 /)\n");
    fprintf(fp, "fill_colours = "
            "(/ 253,243,234,224,215,205,195,186,176,166,157,147,138,128,0,"
            "127,117,108,98,89,79,69,60,50,40,31,21,12,2 /)\n");
    fprintf(fp, "res@lbLabelAngleF = 90.0\n");
  }
}


void PrecipBenchmarkPlot::scaling
(float &scale_factor, float &offset, string &units)
{
  // Input data is in kg m-2 s-1; plots should be in mm day-1.
  offset = 0.0;
  scale_factor = 3600 * 24;
  units = "mm/day";
}


string PrecipBenchmarkPlot::title_string(void)
{
  char buff[256];
  if (_component != ZONAL_AFRICA) {
    string season_string;
    switch(_component) {
    case ANNUAL: season_string = "Annual"; break;
    case DJF: season_string = "DJF"; break;
    case JJA: season_string = "JJA"; break;
    }

    sprintf(buff, "%s %s %s (%s)",
            _model.c_str(), season_string.c_str(), _var_full_name.c_str(),
            string(_setup).c_str());
  } else
    sprintf(buff, "%s N. African Zonal Average %s (%s)",
            _model.c_str(), _var_full_name.c_str(), string(_setup).c_str());
   
  return buff;
}


//----------------------------------------------------------------------
//----------------------------------------------------------------------
//----------------------------------------------------------------------
