//----------------------------------------------------------------------
//
//  FILE:   motif-plots.cpp
//  AUTHOR: Ian Ross
//  DATE:   28-MAR-2005
//
//----------------------------------------------------------------------
//
//  Main program for MOTIF benchmarking plot document generator.
//
//----------------------------------------------------------------------

//----------------------------------------------------------------------
//
//  HEADER FILES
//

// Standard headers.

#include <iostream>
#include <cstdlib>
#include <string>

using namespace std;


// Local headers.

#include "PlotSetup.hh"
#include "SeasonalBenchmarkPlot.hh"
#include "PrecipBenchmarkPlot.hh"
#include "PMinusEBenchmarkPlot.hh"
#include "TempBenchmarkPlot.hh"
#include "IceBenchmarkPlot.hh"
#include "PlotCollator.hh"
#include "TempFile.hh"


//----------------------------------------------------------------------
//
//  MAIN PROGRAM
//

int main(int argc, char *argv[])
{
  // There should just be one command line argument - the model to
  // deal with.

  if (argc != 2) {
    cerr << "Usage: motif-plots <model>" << endl;
    exit(1);
  }
  string model = argv[1];

  try {
    // Set up plot configuration list.

    vector<PlotSetup> setups_modern;
    vector<PlotSetup> setups_6k;
    vector<PlotSetup> setups_21k;
    setups_modern.push_back(PlotSetup(OA_0K));
    setups_modern.push_back(PlotSetup(OAV_0K));
    setups_6k.push_back(PlotSetup(OA_6K));
    setups_6k.push_back(PlotSetup(OAV_6K));
    setups_21k.push_back(PlotSetup(OA_21K));
    setups_21k.push_back(PlotSetup(OAV_21K));
    setups_modern.push_back(PlotSetup(OAV_0K, OA_0K));
    setups_6k.push_back(PlotSetup(OA_6K, OA_0K));
    setups_21k.push_back(PlotSetup(OA_21K, OA_0K));
    setups_6k.push_back(PlotSetup(OAV_6K, OAV_0K));
    setups_21k.push_back(PlotSetup(OAV_21K, OAV_0K));


    // Set up plot generation objects.

    vector<BenchmarkPlot *> plots;
    PlotSetup *last_setup;
    PlotCollator collate(string("benchmark-") + model + ".pdf",
                         model, "Seasonal Benchmarking Plots");


    //------------------------------------------------------------
    //
    //  PRECIPITATION PLOTS
    //
    //------------------------------------------------------------

    collate.new_variable("Total Precipitation");
    for (vector<PlotSetup>::iterator sit = setups_modern.begin();
         sit != setups_modern.end(); ++sit) {
      PlotSetup setup = *sit;
      PrecipBenchmarkPlot *ann =
        new PrecipBenchmarkPlot(model, setup, PrecipBenchmarkPlot::ANNUAL);
      ann->plot();
      PrecipBenchmarkPlot *djf =
        new PrecipBenchmarkPlot(model, setup, PrecipBenchmarkPlot::DJF);
      djf->plot();
      PrecipBenchmarkPlot *jja =
        new PrecipBenchmarkPlot(model, setup, PrecipBenchmarkPlot::JJA);
      jja->plot();
      if (ann->valid()) {
        collate.new_setup(*sit);
        plots.push_back(ann);  collate.add_plot(ann);
        plots.push_back(djf);  collate.add_plot(djf);
        plots.push_back(jja);  collate.add_plot(jja);
      }
    }
    for (vector<PlotSetup>::iterator sit = setups_6k.begin();
         sit != setups_6k.end(); ++sit) {
      PlotSetup setup = *sit;
      PrecipBenchmarkPlot *ann =
        new PrecipBenchmarkPlot(model, setup, PrecipBenchmarkPlot::ANNUAL);
      ann->plot();
      PrecipBenchmarkPlot *djf =
        new PrecipBenchmarkPlot(model, setup, PrecipBenchmarkPlot::DJF);
      djf->plot();
      PrecipBenchmarkPlot *jja =
        new PrecipBenchmarkPlot(model, setup, PrecipBenchmarkPlot::JJA);
      jja->plot();
      PrecipBenchmarkPlot *zonal_africa =
        new PrecipBenchmarkPlot(model, setup,
                                PrecipBenchmarkPlot::ZONAL_AFRICA);
      zonal_africa->plot();
      if (ann->valid()) {
        collate.new_setup(*sit);
        plots.push_back(ann);  collate.add_plot(ann);
        plots.push_back(djf);  collate.add_plot(djf);
        plots.push_back(jja);  collate.add_plot(jja);
        plots.push_back(zonal_africa);  collate.add_plot(zonal_africa, -90);
      }
    }
    for (vector<PlotSetup>::iterator sit = setups_21k.begin();
         sit != setups_21k.end(); ++sit) {
      PlotSetup setup = *sit;
      PrecipBenchmarkPlot *ann =
        new PrecipBenchmarkPlot(model, setup, PrecipBenchmarkPlot::ANNUAL);
      ann->plot();
      PrecipBenchmarkPlot *djf =
        new PrecipBenchmarkPlot(model, setup, PrecipBenchmarkPlot::DJF);
      djf->plot();
      PrecipBenchmarkPlot *jja =
        new PrecipBenchmarkPlot(model, setup, PrecipBenchmarkPlot::JJA);
      jja->plot();
      if (ann->valid()) {
        collate.new_setup(*sit);
        plots.push_back(ann);  collate.add_plot(ann);
        plots.push_back(djf);  collate.add_plot(djf);
        plots.push_back(jja);  collate.add_plot(jja);
      }
    }


    //------------------------------------------------------------
    //
    //  2m AIR TEMPERATURE
    //
    //------------------------------------------------------------

    collate.new_variable("2m Air Temperature");
    for (vector<PlotSetup>::iterator sit = setups_modern.begin();
         sit != setups_modern.end(); ++sit) {
      PlotSetup setup = *sit;
      TempBenchmarkPlot *djf =
        new TempBenchmarkPlot(model, setup, TempBenchmarkPlot::DJF);
      djf->plot();
      TempBenchmarkPlot *jja =
        new TempBenchmarkPlot(model, setup, TempBenchmarkPlot::JJA);
      jja->plot();
      if (djf->valid()) {
        collate.new_setup(*sit);
        plots.push_back(djf);  collate.add_plot(djf);
        plots.push_back(jja);  collate.add_plot(jja);
      }
    }
    for (vector<PlotSetup>::iterator sit = setups_6k.begin();
         sit != setups_6k.end(); ++sit) {
      PlotSetup setup = *sit;
      TempBenchmarkPlot *djf =
        new TempBenchmarkPlot(model, setup, TempBenchmarkPlot::DJF);
      djf->plot();
      TempBenchmarkPlot *jja =
        new TempBenchmarkPlot(model, setup, TempBenchmarkPlot::JJA);
      jja->plot();
      if (djf->valid()) {
        collate.new_setup(*sit);
        plots.push_back(djf);  collate.add_plot(djf);
        plots.push_back(jja);  collate.add_plot(jja);
      }
    }
    for (vector<PlotSetup>::iterator sit = setups_21k.begin();
         sit != setups_21k.end(); ++sit) {
      PlotSetup setup = *sit;
      TempBenchmarkPlot *djf =
        new TempBenchmarkPlot(model, setup, TempBenchmarkPlot::DJF);
      djf->plot();
      TempBenchmarkPlot *jja =
        new TempBenchmarkPlot(model, setup, TempBenchmarkPlot::JJA);
      jja->plot();
//       TempBenchmarkPlot *zonal_land =
//         new TempBenchmarkPlot(model, setup,
//                                 TempBenchmarkPlot::ZONAL_TROP_LAND);
//       zonal_land->plot();
//       TempBenchmarkPlot *zonal_sea =
//         new TempBenchmarkPlot(model, setup,
//                                 TempBenchmarkPlot::ZONAL_TROP_SEA);
//       zonal_sea->plot();
      if (djf->valid()) {
        collate.new_setup(*sit);
        plots.push_back(djf);  collate.add_plot(djf);
        plots.push_back(jja);  collate.add_plot(jja);
//         plots.push_back(zonal_land);  collate.add_plot(zonal_land);
//         plots.push_back(zonal_sea);  collate.add_plot(zonal_sea);
      }
    }


    //------------------------------------------------------------
    //
    //  PRECIPITATION MINUS EVAPORATION
    //
    //------------------------------------------------------------

    collate.new_variable("Precipitation Minus Evaporation");
    for (vector<PlotSetup>::iterator sit = setups_modern.begin();
         sit != setups_modern.end(); ++sit) {
      PlotSetup setup = *sit;
      PMinusEBenchmarkPlot *ann = new PMinusEBenchmarkPlot(model, setup);
      ann->plot();
      if (ann->valid()) {
        collate.new_setup(*sit);
        plots.push_back(ann);
        collate.add_plot(ann);
      }
    }
    for (vector<PlotSetup>::iterator sit = setups_6k.begin();
         sit != setups_6k.end(); ++sit) {
      PlotSetup setup = *sit;
      PMinusEBenchmarkPlot *ann = new PMinusEBenchmarkPlot(model, setup);
      ann->plot();
      if (ann->valid()) {
        collate.new_setup(*sit);
        plots.push_back(ann);
        collate.add_plot(ann);
      }
    }
    for (vector<PlotSetup>::iterator sit = setups_21k.begin();
         sit != setups_21k.end(); ++sit) {
      PlotSetup setup = *sit;
      PMinusEBenchmarkPlot *ann = new PMinusEBenchmarkPlot(model, setup);
      ann->plot();
      if (ann->valid()) {
        collate.new_setup(*sit);
        plots.push_back(ann);
        collate.add_plot(ann);
      }
    }


    //------------------------------------------------------------
    //
    //  SOIL MOISTURE
    //
    //------------------------------------------------------------

    collate.new_variable("Soil Moisture");
    int cnt = 0;
    for (vector<PlotSetup>::iterator sit = setups_modern.begin();
         sit != setups_modern.end(); ++sit) {
      PlotSetup setup = *sit;
      SeasonalBenchmarkPlot *jja =
        new SeasonalBenchmarkPlot(model, SURFACE, "mrso", "Soil Moisture",
                                  setup, SeasonalBenchmarkPlot::JJA);
      jja->plot();
      if (jja->valid()) {
        collate.new_setup(*sit);
        plots.push_back(jja);
        collate.add_plot(jja);
      }
    }
    cnt = 0;
    for (vector<PlotSetup>::iterator sit = setups_6k.begin();
         sit != setups_6k.end(); ++sit) {
      PlotSetup setup = *sit;
      SeasonalBenchmarkPlot *jja =
        new SeasonalBenchmarkPlot(model, SURFACE, "mrso", "Soil Moisture",
                                  setup, SeasonalBenchmarkPlot::JJA);
      jja->plot();
      if (jja->valid()) {
        collate.new_setup(*sit);
        plots.push_back(jja);
        collate.add_plot(jja);
      }
    }
    cnt = 0;
    for (vector<PlotSetup>::iterator sit = setups_21k.begin();
         sit != setups_21k.end(); ++sit) {
      PlotSetup setup = *sit;
      SeasonalBenchmarkPlot *jja =
        new SeasonalBenchmarkPlot(model, SURFACE, "mrso", "Soil Moisture",
                                  setup, SeasonalBenchmarkPlot::JJA);
      jja->plot();
      if (jja->valid()) {
        collate.new_setup(*sit);
        plots.push_back(jja);
        collate.add_plot(jja);
      }
    }


    //------------------------------------------------------------
    //
    //  SEA ICE
    //
    //------------------------------------------------------------

    collate.new_variable("Sea Ice Concentration");
    for (vector<PlotSetup>::iterator sit = setups_modern.begin();
         sit != setups_modern.end(); ++sit) {
      PlotSetup setup = *sit;
      IceBenchmarkPlot *jjan =
        new IceBenchmarkPlot(model, setup, IceBenchmarkPlot::JJA,
                             IceBenchmarkPlot::NORTH);
      jjan->plot();
      IceBenchmarkPlot *jjas =
        new IceBenchmarkPlot(model, setup, IceBenchmarkPlot::JJA,
                             IceBenchmarkPlot::SOUTH);
      jjas->plot();
      IceBenchmarkPlot *djfn =
        new IceBenchmarkPlot(model, setup, IceBenchmarkPlot::DJF,
                             IceBenchmarkPlot::NORTH);
      djfn->plot();
      IceBenchmarkPlot *djfs =
        new IceBenchmarkPlot(model, setup, IceBenchmarkPlot::DJF,
                             IceBenchmarkPlot::SOUTH);
      djfs->plot();
      if (jjan->valid()) {
        collate.new_setup(*sit);
        plots.push_back(jjan);  collate.add_plot(jjan, 90.0);
        plots.push_back(jjas);  collate.add_plot(jjas, 90.0);
        plots.push_back(djfn);  collate.add_plot(djfn, 90.0);
        plots.push_back(djfs);  collate.add_plot(djfs, 90.0);
      }
    }
    for (vector<PlotSetup>::iterator sit = setups_21k.begin();
         sit != setups_21k.end(); ++sit) {
      PlotSetup setup = *sit;
      IceBenchmarkPlot *jjan =
        new IceBenchmarkPlot(model, setup, IceBenchmarkPlot::JJA,
                             IceBenchmarkPlot::NORTH);
      jjan->plot();
      IceBenchmarkPlot *jjas =
        new IceBenchmarkPlot(model, setup, IceBenchmarkPlot::JJA,
                             IceBenchmarkPlot::SOUTH);
      jjas->plot();
      IceBenchmarkPlot *djfn =
        new IceBenchmarkPlot(model, setup, IceBenchmarkPlot::DJF,
                             IceBenchmarkPlot::NORTH);
      djfn->plot();
      IceBenchmarkPlot *djfs =
        new IceBenchmarkPlot(model, setup, IceBenchmarkPlot::DJF,
                             IceBenchmarkPlot::SOUTH);
      djfs->plot();
      if (jjan->valid()) {
        collate.new_setup(*sit);
        plots.push_back(jjan);  collate.add_plot(jjan, 90.0);
        plots.push_back(jjas);  collate.add_plot(jjas, 90.0);
        plots.push_back(djfn);  collate.add_plot(djfn, 90.0);
        plots.push_back(djfs);  collate.add_plot(djfs, 90.0);
      }
    }


    //------------------------------------------------------------
    //
    //  SEA SURFACE TEMPERATURE
    //
    //------------------------------------------------------------

    collate.new_variable("Sea Surface Temperature");
    for (vector<PlotSetup>::iterator sit = setups_modern.begin();
         sit != setups_modern.end(); ++sit) {
      PlotSetup setup = *sit;
      SeasonalBenchmarkPlot *ann =
        new SeasonalBenchmarkPlot(model, OCEAN, "tos",
                                  "Sea Surface Temperature", setup,
                                  SeasonalBenchmarkPlot::ANNUAL);
      ann->plot();
      SeasonalBenchmarkPlot *djf =
        new SeasonalBenchmarkPlot(model, OCEAN, "tos",
                                  "Sea Surface Temperature", setup,
                                  SeasonalBenchmarkPlot::DJF);
      djf->plot();
      SeasonalBenchmarkPlot *jja =
        new SeasonalBenchmarkPlot(model, OCEAN, "tos",
                                  "Sea Surface Temperature", setup,
                                  SeasonalBenchmarkPlot::JJA);
      jja->plot();
      if (ann->valid()) {
        collate.new_setup(*sit);
        plots.push_back(ann);  collate.add_plot(ann);
        plots.push_back(djf);  collate.add_plot(djf);
        plots.push_back(jja);  collate.add_plot(jja);
      }
    }
    for (vector<PlotSetup>::iterator sit = setups_21k.begin();
         sit != setups_21k.end(); ++sit) {
      PlotSetup setup = *sit;
      SeasonalBenchmarkPlot *ann =
        new SeasonalBenchmarkPlot(model, OCEAN, "tos",
                                  "Sea Surface Temperature", setup,
                                  SeasonalBenchmarkPlot::ANNUAL);
      ann->plot();
      SeasonalBenchmarkPlot *djf =
        new SeasonalBenchmarkPlot(model, OCEAN, "tos",
                                  "Sea Surface Temperature", setup,
                                  SeasonalBenchmarkPlot::DJF);
      djf->plot();
      SeasonalBenchmarkPlot *jja =
        new SeasonalBenchmarkPlot(model, OCEAN, "tos",
                                  "Sea Surface Temperature", setup,
                                  SeasonalBenchmarkPlot::JJA);
      jja->plot();
      if (ann->valid()) {
        collate.new_setup(*sit);
        plots.push_back(ann);  collate.add_plot(ann);
        plots.push_back(djf);  collate.add_plot(djf);
        plots.push_back(jja);  collate.add_plot(jja);
      }
    }


    // Collate plots.

    collate();


    for (vector<BenchmarkPlot *>::iterator it = plots.begin();
         it != plots.end(); ++it)
      delete *it;
    plots.clear();
    TempFile::clean();
  } catch (const char *msg) {
    cout << "EXCEPTION: " << msg << endl;
  }
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//----------------------------------------------------------------------

